/*
 * This file is a part of CMSLU -- Content Management System like Unix
 * and is not publicly distributed
 * 
 * Copyright (c) 2008-2009, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#ifndef headerfilecmslutemplatestemplates
#define headerfilecmslutemplatestemplates

#include <iomanip>
#include <set>

#include "../../ezc/src/ezc.h"
#include "patterncacher.h"


#ifdef APPTEMPLATES
#include "../app/templates.h"
#endif


namespace TemplatesFunctions
{
	using Ezc::Info;
	
	extern PatternCacher pattern_cacher;
	extern Ezc::Functions functions;
	extern Ezc::Pattern pat_item_info;
	extern Ezc::Pattern pat_item_tab_info;

	/*
		sys
	*/
	void sys_ver_major(Info & i);
	void sys_ver_minor(Info & i);
	void sys_ver_revision(Info & i);


	/*
		doc
	*/
	void doc_language(Info & i);
	void doc_title(Info & i);
	void doc_charset(Info & i);
	void doc_base_url(Info & i);
	void doc_current_url(Info & i);
	void doc_status_error(Info & i);
	void doc_status_code(Info & i);


	/*
		item
	*/
	void item_is(Info & i);
	void item_id(Info & i);
	void item_subject(Info & i);
	void item_subject_noescape(Info & i);
	void item_content(Info & i);
	void item_content_noescape(Info & i);
	void item_privileges(Info & i);
	void item_dir(Info & i);  // the same as full_dir()
	void item_url(Info & i);
	void item_link(Info & i);
	void item_can_read(Info & i);
	void item_info(Info & i);
	void item_user(Info & i);
	void item_date_creation(Info & i);
	void item_date_modification(Info & i);
	void item_dates_equal(Info & i);
	void item_run(Info & i);

	// !! wprowadzic item_old_is ?	
	void item_old_id(Info & i);
	void item_old_subject(Info & i);
	void item_old_subject_noescape(Info & i);
	void item_old_content(Info & i);
	void item_old_content_noescape(Info & i);
	void item_old_privileges(Info & i);
	void item_old_dir(Info & i);
	void item_old_url(Info & i);
	void item_old_link(Info & i);

	void item_tab(Info & i);
	void item_tab_id(Info & i);
	void item_tab_subject(Info & i);
	void item_tab_subject_noescape(Info & i);
	void item_tab_content(Info & i);
	void item_tab_content_noescape(Info & i);
	void item_tab_privileges(Info & i);
	void item_tab_dir(Info & i);
	void item_tab_url(Info & i);
	void item_tab_link(Info & i);
	void item_tab_can_read(Info & i);
	void item_tab_info(Info & i);
	void item_tab_user(Info & i);
	void item_tab_date_creation(Info & i);
	void item_tab_date_modification(Info & i);
	void item_tab_dates_equal(Info & i);
	void item_tab_run(Info & i);
	
	
	/*
		dir
	*/
	void dir(Info & i);
	void dir_can_read_exec(Info & i);
	
	void dir_childs_tab(Info & i);
	void dir_childs_tab_url(Info & i);

	void dir_tab(Info & i);
	void dir_tab_url(Info & i);
	void dir_tab_link(Info & i);

	void dir_last_default_item_dir(Info &);
	void dir_last_default_item_url(Info & i);

	void dir_old(Info & i);


	/*
		user
	*/
	void user_name(Info & i);
	void user_logged(Info & i);
	void user_super_user(Info & i);
	
	
	/*
		privileges
	*/
	void priv_user_tab(Info & i);
	void priv_user_tab_name(Info & i);
	void priv_user_tab_isdefault(Info & i);
	
	void priv_group_tab(Info & i);
	void priv_group_tab_name(Info & i);
	void priv_group_tab_isdefault(Info & i);
	
	
	
	/*
		done
	*/
	void done_errors(Info & i);
	void done_status(Info & i);
	void done_status_no_item(Info & i);
	void done_status_incorrect_dir(Info & i);

	void done_added_item(Info & i);
	void done_edited_item(Info & i);
	void done_deleted_item(Info & i);
	void done_privileged_item(Info & i);

	void done_added_dir(Info & i);
	void done_deleted_dir(Info & i);
	void done_defaulted_dir(Info & i);

	void done_loggedout(Info & i);


	/*
		who
	*/
	void who_tab(Info & i);
	void who_tab_lp(Info & i);
	void who_tab_user(Info & i);
	void who_tab_time(Info & i);
	void who_tab_last_time(Info & i);


	/*
		last
	*/
	void last_tab(Info & i);
	void last_tab_name(Info & i);
	void last_tab_ip(Info & i);
	void last_tab_start(Info & i);
	void last_tab_end(Info & i);


	/*
		thread
	*/
	void thread_subject(Info & i);
	
	

} // namespace TemplatesFunctions




class Templates
{


public:


	void Read();
	void Generate();

	void CreateFunctions();
	

};



#endif
